#!/usr/bin/perl
# VRgallery - ein einfacher Ansatz fuer eigene Bildergalerien mit Perl und VRML
# (c) Copyright 1998 Ramon Wartala (wartala@stud.fh-heilbronn.de).
# Galerie Konstruktion und Design (c) 1998 Aria Behbehani (ariab@tin.it).
# Perl VRML Paket (c) Hartmut Palm
# Image::Size Paket (c) Randy Ray
#
# letzte Aenderung 11.06.1998, 02:05 Uhr

use strict;
require 5.000;
use Getopt::Std;  # Paket zur Verarbeitung von Kommandozeilen-Schaltern
use File::Find;   # Paket zur Verarbeitung von Datei-Baeumen
use Image::Size;	# Paket zur Ermittlung von Bildgroessen
use VRML;         # Paket zur Erzeugung von VRML Welten

use vars qw(
	$opt_m $opt_d $opt_g $opt_v $opt_r $dev $ino $mode $nlink $uid
	$gid $rdev $size $atime $mtime $ctime $blksize $blocks
);

my $VERSION="0.9.5"; # VRgellery Versionsnummer
my $startdir = "";   # Start Verzeichnis
my $maximg = 0;      # Maximale Anzahl Bilder
my $ipr = 9;         # Anzahl Bilder pro Raum
my $counter = 0;     # Bilder Index
my $verbose = 0;		 # Mit Ausgabe (1) oder ohne (0)
my $usegzip = 0;     # GZIP Schalter
my @images = ();     # Feld mit allen Bilder Pfaden

# Durchsuche am dem Startverzeichnis alle Verzeichnisse und sichere die Bilder Pfade
sub search_dir {
	if ($counter == $maximg) {
		$File::Find::prune = 0;
	} else {
		if ($File::Find::name =~ '.jpg|.JPG|.jpeg|.JPEG|.gif|.GIF|.png|.PNG') {
			push @images, $File::Find::name;
			$counter++;
		}
	}
}

# Kommandozeilen Optionen ermitteln
sub parse_args {
	if($#ARGV == -1) {
		print "Aufruf: VRgallery\n";
    print "\t-m <nummer> maximale Anzahl verwendeter Bilder\n";
		print "\t-r <nummer> Anzahl Bilder pro Raum (<=9)\n";
		print "\t-d          Start Verzeichnis\n";
    print "\t-g          Soll GZIP zum packen der Szenen benutzt werden\n";
		print "\t-v          verbose\n\n";
		print "Beispiel: VRgallery -d c:\\myimages -m 9 -r 9 -g -v\n";
    exit(-1);
  } else {
    getopts('m:r:vgd:');

	  if($opt_v) { $verbose = 1	}

		if($opt_m) {
			$maximg = $opt_m;
			print "suche nach maximal $maximg Bildern","\n" if $verbose;
		}

		if ($opt_d) {
  		$startdir = $opt_d;
  	} else { $startdir = "." }

		print "durchsuche $startdir","\n" if $verbose;;
		finddepth(\&search_dir,$startdir);
		#find(\&search_dir,$startdir);

		if($opt_g) { $usegzip = 1 }

		if($opt_r) {
			if($opt_r <= $ipr) {
				$ipr = $opt_r
			}

			if($ipr > 1) {
				print "$ipr Bilder pro Raum","\n" if $verbose;
			} else {
				print "nur ein Bild pro Raum","\n" if $verbose;
			}
		}
	}
}

# Positioniert Kameras auf die entsprechenden Bilder
sub pos_camera {
	my ($gallery, $order, $image) = @_;
	my $picname = "";

	my %campos = (
		frontfront => ["0.0 0.0 2.2","0 1 0 -3.142"],
		frontright => ["1.0 0.0 4.0","0 -1 0 -1.8"],
		frontleft  => ["-1.0 0.0 4.0","0 1 0 -1.8"],
		rightfront => ["-2.2 0.0 0.0","0 -1 0 -1.571"],
		rightright => ["-5.0 0.0 2.0","0 -1 0 -0.2"],
		rightleft  => ["-5.0 0.0 -2.0","0 1 0 -3.142"],
		leftfront  => ["2.2 0.0 0.0","0 1 0 -1.571"],
		leftleft   => ["5.0 0.0 2.0","0 -1 0 -0.2"],
		leftright  => ["5.0 0.0 -2.0","0 1 0 -3.142"],
  );

	$gallery->insert("Viewpoint {")
		->insert("  description	\"Picture: $image\"")
		->insert("  position $campos{$order}[0]")
		->insert("  orientation	$campos{$order}[1]")
		->insert("  fieldOfView	0.7")
	->insert("}");
}

sub pos_image {
	my ($gallery, $order, $image) = @_;
	my $width = 0;
	my $height = 0;
	my @q_scale =("1","3","3");
	my @l_scale =("1","3","3.5");
	my @p_scale =("1","4.5","3");
	my $scale = "";

	# Bilderrahmen Position als Hash: trans[0-2], rot[3-6], rot_tex[7]
	my %picframe =(
		rightfront => ["-5.8","-0.3712","-0.02398","0","0","0","-1.571","-3.15"],
		leftfront	 => ["5.99","-0.3712","-0.02398","0","0","0","-1.571","0"],
		frontfront => ["0","-0.3712","5.99","0","1","0","-4.713",-3.15],           # richtig
		frontright => ["-2.65","-0.3937","4.8","0","1","0","0.25","-3.15"],        # richtig
		frontleft  => ["2.625","-0.3937","4.8","0","1","0","-3.37","-3.15"],       # richtig
		rightright => ["-4.761","-0.3937","-2.6","0","1","0","-1.326","-3.15"],    # richtig
		rightleft  => ["-4.761","-0.3937","2.7","0","1","0","1.326","-3.15"],      # richtig
		leftright  => ["4.745","-0.3937","2.65","0","1","0","-1.326","0"],
		leftleft   => ["4.745","-0.3937","-2.65","0","1","0","-1.812","-3.15"],    # richtig
	);

	# Bildhoehe und Bildbreite zur Bestimmung der Rahmenproportionen ermitteln
	($width,$height) = imgsize($image);

	if ($width > $height) {
		$scale = "scal $l_scale[0] $l_scale[1] $l_scale[2]";
	} elsif ($width == $height) {
		$scale = "scal $q_scale[0] $q_scale[1] $q_scale[2]";
	} else {
		$scale = "scal $p_scale[0] $p_scale[1] $p_scale[2]";
	}

	# Das Bild und der Link auf die orginal Datei
	$gallery->comment("Bilderrahmen mit Bild $image");
	$gallery->insert("Anchor {url \"$image\" parameter \"target=_blank\" children [")
		->insert("PictureFrame {
			trans $picframe{$order}[0] $picframe{$order}[1] $picframe{$order}[2]
			rot $picframe{$order}[3] $picframe{$order}[4] $picframe{$order}[5] $picframe{$order}[6]
			$scale
			picture_tex \"$image\"
			rot_tex $picframe{$order}[7]
		}")
	->insert("]}");

	# Kameras mit direkter Sicht auf die Bilder positionieren
	&pos_camera($gallery,$order,$image);
}

# Erzeugt die Menue- und den Tour-Button
sub create_menu {
	my($gallery,$no_of_rooms,$actual_room) = @_;
	my $i = 0;
	my $ypos = -2;

	$gallery->comment("Rundgang TouchSensor");

	$gallery->transform_begin("t=0.0 -2.5 -10.0")
		->billboard_begin("0 1 0",undef,"0 0 0")
			->touchsensor("TourButton")
			->transform_begin("t=0.0 0.0 -0.1")
				->box("1.5 0.5 0.1","d=0.6314 0.6314 0.6314;sh=0.92;tr=0.5")
			->transform_end
			->transform_begin("t=-0.6 -0.1 0.0")
				->text("tour","white","0.5 SERIF BOLD","BEGIN")
			->transform_end
		->billboard_end
	->transform_end;

	$gallery->route("TourButton.touchTime","TourClock.startTime");
	$gallery->route("TourButton.touchTime","TurnClock.startTime");
	$gallery->route("TourClock.fraction_changed","TourPos.set_fraction");
	$gallery->route("TurnClock.fraction_changed","TourOri.set_fraction");
	$gallery->route("TourOri.value_changed","TourEye.set_orientation");
	$gallery->route("TourPos.value_changed","TourCam.set_translation");

	$gallery->comment("Menue mit Links zu den einzelnen Raeumen");

	for($i=1;$i <= $no_of_rooms;$i++) {
		$gallery->transform_begin("t=0.0 $ypos -10.0")
			->billboard_begin("0 1 0",undef,"0 0 0")
				->anchor_begin("room$i.html","room$i.wrl",undef,undef,"0 0 0")
					->transform_begin("t=0.0 0.0 -0.1")
						->box("1.5 0.5 0.1","d=0.6314 0.6314 0.6314;sh=0.92;tr=0.5")
					->transform_end
					->transform_begin("t=-0.6 -0.1 0.0");
						if($i == $actual_room) {
							$gallery->interpolator("BlinkInterpol","Color",[0,1],["0.6 0.6 0.6","0.9 0.7 0.02"]);
							$gallery->timesensor("BlinkTime","3","TRUE");

							$gallery->insert("Shape { appearance Appearance { material DEF TextMaterial Material {
								diffuseColor	1 1 1	}}geometry Text {	string \"room $i\" fontStyle FontStyle { size 0.5
								family \"SERIF\" style \"BOLD\"	justify \"BEGIN\"	}	}	} # Shape");
						}	else {
							$gallery->text("room $i","white","0.5 SERIF BOLD","BEGIN");
						}

					$gallery->transform_end
				->anchor_end
			->billboard_end
		->transform_end;

		if($i == $actual_room) {
			$gallery->route("BlinkTime.fraction_changed","BlinkInterpol.set_fraction");
			$gallery->route("BlinkInterpol.value_changed","TextMaterial.set_diffuseColor");
		}

		$ypos=$ypos+0.5
	}
}

# Erzeugt die eigentlich Galerie
sub create_gallery {
	my $no_of_pics = 0;		 					 # Anzahl Bilder
	my $gallery = undef; 						 # VRML Objekt
	my $no_of_rooms = 0;						 # Anzahl der Raeume
	my $nor = 0.0;
	my $nor2 = 0;
	my $image = "";									 # Aktuelles Bild
	my $ic = 0;											 # Bilder Index
	my $gc = 0;											 # Galerie Index
	my $rest = 0;
	my $order_idx = 0;							 # Reihenfolge Index
	my $i = 0;
	my $j = 0;

	# Reihenfolge des Aufhaengens der Bilderrahmen
	my @picord =("frontfront","rightfront","leftfront",
		"frontleft","frontright","leftleft","leftright",
		"rightleft","rightright","exitleft","exitright"
	);

	$no_of_pics = scalar @images;
	print "$no_of_pics Bilder gefunden\n" if $verbose;

	# Wieviel Raeume soll die Galerie haben?
	$nor= $no_of_pics / $ipr;
	$nor2 = int($nor);
	if(abs($nor) > $nor2) {$nor2++}
  $no_of_rooms = $nor2;

	if ($no_of_rooms > 1) {
		print "Erzeuge $no_of_rooms Raeume fuer die Galerie...","\n" if $verbose;
	} else {
		print "Erzeuge nur einen Raum fuer die Galerie...","\n" if $verbose;
	}

	$rest = $no_of_pics;

	for($j=1;$j <= $no_of_rooms;$j++) {
		$gc++;
		$order_idx = 0;
		print "Erzeuge Raum Nr. $gc\n" if $verbose;

		$gallery = new VRML(2);
		$gallery->comment("Diese VRML 2.0 Datei wurde mit VRgallery $VERSION erzeugt, (c) 1998 by R. Wartala, A. Behbehani");
		$gallery->comment("");
		$gallery->browser("CosmoPlayer","Microsoft Internet Explorer");
		$gallery->title("Raum $gc, erzeugt mit VRgallery $VERSION - (c) 1998 by R. Wartala, A. Behbehani");
		$gallery->navigationinfo("ANY",1.0,0,0.0,"0.25 1.8 0.75");

		# Mr. Behbehani's Galerie einbinden
		$gallery->include("vrgallery.wrl");

		if($rest > $ipr) {
			for($i=1;$i <= $ipr;$i++) {
				print "$ic. Bild $images[$ic] fuer Galerieraum $gc\n" if $verbose;
				&pos_image($gallery,$picord[$order_idx],$images[$ic]);
				$ic++;
				$order_idx++;
				if ($order_idx == 9) {
					$order_idx = 0;
				}
				$rest--;
			}
		} else {
			for($i=1;$i <= $rest;$i++) {
				print "$ic. Bild $images[$ic] fuer Galerieraum $gc\n" if $verbose;
				&pos_image($gallery,$picord[$order_idx],$images[$ic]);
				$ic++;
				$order_idx++;
				if ($order_idx == 9) {
					$order_idx = 0;
				}
			}
		}
		
		# Raum Auswahlliste erzeugen
		&create_menu($gallery,$no_of_rooms,$gc);

		# VRML Quellcode speichern
		if($usegzip == 1) {
			$gallery->save("room$gc.wrl","gzip");
		} else {
			$gallery->save("room$gc.wrl");
		}

		# HTML Seite mit eingebetteter VRgallery erzeugen
		open(HTML,">room$gc.html") or die "can't create html dummy!\n";
		print HTML "<HTML><HEAD><TITLE>VRgallery: room$gc</TITLE></HEAD>\n";
		print HTML "<BODY LINK=\"#f9bc09\" TEXT=\"#aaaaaa\" BGCOLOR=\"#000000\"><CENTER><TABLE BORDER=\"0\"><TR><TD>\n";
		print HTML "<EMBED SRC=\"room$gc.wrl\" HEIGHT=\"480\" WIDTH=\"640\">\n";
		print HTML "</TD></TR>\n";
		print HTML "<TR ALIGN=\"center\" VALIGN=\"middle\"><TD><FONT SIZE=\"2\"><I>Created with VRgallery $VERSION";
		print HTML ", &copy copyright 1998 by <A HREF=\"mailto:wartala\@stud.fh-heilbronn.de\">R. Wartala</A>,";
		print HTML " <A HREF=\"mailto:ariab\@tin.it\">A. Behbehani.</A></FONT>\n";
		print HTML "</TABLE></CENTER></BODY></HTML>\n";
		close HTML;
	}
}

# Hier gehts los...
print "VRgallery ",$VERSION," - (c) 1998 by R. Wartala, A. Behbehani","\n";
&parse_args();
&create_gallery();

